@@ -1,4 +1,4 @@
-from typing import Any, List, Mapping, Optional, Tuple, Type
+from typing import Any, List, Mapping, Optional, Tuple, Type, Union
 
 import docutils
 from docutils import nodes
@@ -12,7 +12,7 @@ from sphinx.transforms.post_transforms import SphinxPostTransform
 from sphinx.util import logging
 from sphinx.util.nodes import clean_astext
 
-from .nodes import HiddenCellNode, RootHeader
+from .nodes import CellInput, CellOutput, HiddenCellNode, RootHeader
 
 logger = logging.getLogger(__name__)
 
@@ -30,12 +30,38 @@ def replace_node_cls(
     src_node.replace_self([node])
 
 
+def findall(node: nodes.Element):
+    # findall replaces traverse in docutils v0.18
+    # note a difference is that findall is an iterator
+    # taken from MyST-NB
+    return getattr(node, "findall", node.traverse)
+
+
+def check_dependency() -> Union[bool, dict]:
+    """Check installed packages and their compatible versions.
+    At present only concerned with MyST-NB"""
+    try:
+        from myst_nb import __version__
+    except ImportError:
+        return False
+    major, minor = __version__.split(".")[0:2]
+    if int(major) == 0 and 13 <= int(minor) < 18:  # TODO: fetch this from setup.cfg?
+        package_versions = {"myst_nb": minor}
+        return package_versions
+    else:
+        logger.warning(
+            "[sphinx-jupyterbook-latex]: myst-nb version not compatible with >=0.13,<0.18: "
+            f"{__version__}"
+        )
+    return False
+
+
 def find_parent(
     env: BuildEnvironment, node: docutils.nodes.Element, parent_tag: str
 ) -> Optional[docutils.nodes.Element]:
     """Find the parent node."""
     while True:
-        node = node.parent
+        node = node.parent  # type: ignore[assignment]
         if node is None:
             return None
         # parent should be a document in toc
@@ -93,7 +119,7 @@ class LatexRootDocTransforms(SphinxTransform):
         # add the docname and header_level attributes to section nodes
         # so we can identify them later, when LatexBuilder merges the doctrees
         def _recursive_assign_depth(
-            node: docutils.nodes.Element, section_depth: int
+            node: docutils.nodes.Node, section_depth: int
         ) -> None:
             for child in node.children:
                 if isinstance(child, docutils.nodes.section):
@@ -111,37 +137,59 @@ class MystNbPostTransform(SphinxPostTransform):
     default_priority = 400
 
     @classmethod
-    def check_dependency(cls) -> bool:
-        """Check that myst-nb is installed and a compatible version."""
-        try:
-            from myst_nb import __version__
-        except ImportError:
-            return False
-        major, minor = __version__.split(".")[0:2]
-        if major == "0" and minor in (
-            "11",
-            "12",
-            "13",
-        ):  # TODO: fetch this from setup.cfg?
-            return True
-        else:
-            logger.warning(
-                "[sphinx-jupyterbook-latex]: myst-nb version not compatible with >=0.11,<0.14: "
-                f"{__version__}"
-            )
+    def check_mystnb_dependency(cls) -> Union[bool, int]:
+        dependencies = check_dependency()
+        if isinstance(dependencies, dict):
+            return int(dependencies.get("myst_nb", ""))
         return False
 
     def apply(self, **kwargs: Any) -> None:
-        from myst_nb.nodes import CellInputNode, CellNode, CellOutputNode
 
-        for node in self.document.traverse(CellNode):
+        mystnb_version = self.check_mystnb_dependency()
+
+        # checking mystnb_version for proper imports
+        if mystnb_version < 14:
+            from myst_nb.nodes import CellInputNode, CellNode, CellOutputNode
+
+            node_search = CellNode
+        else:
+
+            def node_search(node):
+                return isinstance(node, nodes.container)
+
+        for node in list(findall(self.document)(node_search)):
             if "tag_hide-cell" in node["classes"]:
                 replace_node_cls(node, HiddenCellNode, True)
             if "tag_hide-input" in node["classes"]:
-                for input_node in node.traverse(CellInputNode):
+                # checking mystnb_version for proper node search
+                # as myst-nb has started using containers for code cells
+                # from v14
+                if mystnb_version < 14:
+                    node_search = CellInputNode
+                else:
+
+                    def node_search(node):
+                        return isinstance(
+                            node, nodes.container
+                        ) and "cell_input" in node.attributes.get("classes", "")
+
+                for input_node in list(findall(node)(node_search)):
                     replace_node_cls(input_node, HiddenCellNode, True)
+
             if "tag_hide-output" in node["classes"]:
-                for output_node in node.traverse(CellOutputNode):
+                # checking mystnb_version for proper node search
+                # as myst-nb has started using containers for code cells
+                # from v14
+                if mystnb_version < 14:
+                    node_search = CellOutputNode
+                else:
+
+                    def node_search(node):
+                        return isinstance(
+                            node, nodes.container
+                        ) and "cell_output" in node.attributes.get("classes", "")
+
+                for output_node in list(findall(node)(node_search)):
                     replace_node_cls(output_node, HiddenCellNode, True)
 
 
@@ -248,7 +296,7 @@ class LatexRootDocPostTransforms(SphinxPostTransform):
         #       <compound classes="toctree-wrapper">
         #          ...
 
-        if self.env.jblatex_captions_to_parts:  # type: ignore[attr-defined]
+        if self.env.jblatex_captions_to_parts:
 
             for node in self.document.traverse(docutils.nodes.compound):
                 if (
@@ -308,7 +356,7 @@ class LatexToctreeNodeInterpret(SphinxTransform):
 
             # copy all toctree attributes
             hiddennode = HiddenCellNode()
-            hiddennode.attributes = tocnode.attributes
+            hiddennode.attributes = tocnode.attributes  # type: ignore[attr-defined]
             hiddennode.attributes["classes"] = "latex-tableofcontents"
 
             # inserts hiddennode adjacent to a toctree node
@@ -420,3 +468,59 @@ class ListTableOfContents(SphinxPostTransform):
                         [tocnode], nodes_to_visit, inserted_nodes
                     )
                     tocnode.replace_self(listnode)
+
+
+class CodeBlockTransforms(SphinxPostTransform):
+    """Handling any post transforms needed for code cells."""
+
+    default_priority = 999
+
+    @classmethod
+    def check_mystnb_dependency(cls) -> Union[bool, int]:
+        dependencies = check_dependency()
+        if isinstance(dependencies, dict):
+            return int(dependencies.get("myst_nb", ""))
+        return False
+
+    def apply(self):
+        if isinstance(self.env.app.builder, builders.latex.LaTeXBuilder):
+            """Wrapping myst_nb code cell nodes with nodes of this extension."""
+            mystnb_version = self.check_mystnb_dependency()
+
+            # checking mystnb_version for proper imports
+            # as myst-nb has started using containers for code cells
+            # from v14
+            if mystnb_version < 14:
+                from myst_nb.nodes import CellInputNode
+
+                node_search = CellInputNode
+            else:
+
+                def node_search(node):
+                    return isinstance(
+                        node, nodes.container
+                    ) and "cell_input" in node.attributes.get("classes", "")
+
+            for node in list(findall(self.document)(node_search)):
+                cellinput = CellInput()
+                cellinput.append(node.deepcopy())
+                node.replace_self(cellinput)
+
+            # checking mystnb_version for proper imports
+            # as myst-nb has started using containers for code cells
+            # from v14
+            if mystnb_version < 14:
+                from myst_nb.nodes import CellOutputNode
+
+                node_search = CellOutputNode
+            else:
+
+                def node_search(node):
+                    return isinstance(
+                        node, nodes.container
+                    ) and "cell_output" in node.attributes.get("classes", "")
+
+            for node in list(findall(self.document)(node_search)):
+                celloutput = CellOutput()
+                celloutput.append(node.deepcopy())
+                node.replace_self(celloutput)
